/*
 * Special Validator for date for credit card
 * by PhilFlash
 */
package
{
	import mx.validators.IValidatorListener;
	import mx.validators.ValidationResult;
	import mx.validators.Validator;

	public class CreditDateValidator extends Validator
	{
		/**
		 *  Constructor.
		 */
		public function CreditDateValidator()
		{
			super();		
			// Nécessaire pour ré-initialiser les erreurs
			subFields = [ "month", "year" ];
		}
		
		override protected function doValidation(value:Object) : Array
		{
			//var result:Array = super.doValidation(value);
			var results:Array = new Array();

			var year:Number = parseInt(value.year);
			var month:Number = parseInt(value.month);
			
			if (isNaN(year)) year = 0;
			if (isNaN(month)) month = 0;
			
			if (required) 
			{

				if (month == 0) {
					results.push( new ValidationResult(true,"month","1030","A month is required for this date") );
				}
				if (year == 0) {
					results.push( new ValidationResult(true,"year","1040","A year is required for this date") );
				}
			}
			else 
			// not required but only one data => error
			{
				if (month == 0 && year != 0) {
					results.push( new ValidationResult(true,"month","1050","A month is required for this date") );
				}	
				if (month != 0 && year == 0) {
					results.push( new ValidationResult(true,"year","1060","A year is required for this date") );
				}			
			}
			// check date (expired)
			var d:Date = new Date();
			var currentMonth:Number = d.getMonth()+1;
			var currentYear:Number = d.getFullYear();
			if (year < currentYear)
			{
				results.push( new ValidationResult(true,"month","1050","Your credit card has expired") );	
			}
			else
			{
				if (year == currentYear && month < currentMonth)
				{
					results.push( new ValidationResult(true,"month","1050","Your credit card has expired") );	
				}
			}
			return results;
		}
		
		//----------------------------------
		//  actualListeners
		//----------------------------------
	
		/** 
		 *  @private
		 *  Returns either the listener or the source
		 *  for the day, month and year subfields.
		 */
		override protected function get actualListeners():Array
		{
			var results:Array = [];
		
			var monthResult:Object;
			if (_monthListener)
				monthResult = _monthListener;
				
			if (monthResult)
			{
				results.push(monthResult);
				if (monthResult is IValidatorListener)
					IValidatorListener(monthResult).validationSubField = "month";
			}
			
			var yearResult:Object;
			if (_yearListener)
				yearResult = _yearListener;
				
			if (yearResult)
			{
				results.push(yearResult);
				if (yearResult is IValidatorListener)
					IValidatorListener(yearResult).validationSubField = "year";
			}

			
			if (results.length > 0 && listener)
			{
				results.push(listener);
			}
			else
			{
				// Pas nécessaire -> sinon on prend la source (ce qui décale le message)
				//results = results.concat(super.actualListeners);
			}
			
			return results;
		}
		
		//----------------------------------
		//  monthListener
		//----------------------------------		
		
		/**
		 *  @private
		 *  Storage for the monthListener property.
		 */
		private var _monthListener:IValidatorListener; 
		
		[Inspectable(category="General")]
	
		/** 
		 *  The component that listens for the validation result
		 *  for the month subfield. 
		 *  If none is specified, use the value specified
		 *  for the <code>monthSource</code> property.
		 */
		public function get monthListener():IValidatorListener
		{
			return _monthListener;
		}
		
		/**
		 *  @private
		 */
		public function set monthListener(value:IValidatorListener):void
		{
			if (_monthListener == value)
				return;
				
			removeListenerHandler();	
				
			_monthListener = value;
			
			addListenerHandler();
		}

		//----------------------------------
		//  yearListener
		//----------------------------------
		
		/**
		 *  @private
		 *  Storage for the yearListener property.
		 */
		private var _yearListener:IValidatorListener; 
		
		[Inspectable(category="General")]
	
		/** 
		 *  The component that listens for the validation result
		 *  for the year subfield. 
		 *  If none is specified, use the value specified
		 *  for the <code>yearSource</code> property.
		 */
		public function get yearListener():IValidatorListener
		{
			return _yearListener;
		}
		
		/**
		 *  @private
		 */
		public function set yearListener(value:IValidatorListener):void
		{
			if (_yearListener == value)
				return;
				
			removeListenerHandler();	
				
			_yearListener = value;
			
			addListenerHandler();
		}
	}
}
